package at.jku.ssw.graphviz;

import java.util.HashMap;
// http://graphviz.org
import java.util.Map;

import at.jku.ssw.list.ListNode;

/**
 * A maker of dot strings for GraphViz in order to display data structures.
 */
public class DotMaker {

	/**
	 * Creates a dot string for a linked list.
	 */
	public static String makeDotForLinkedList(ListNode head) {
		final Map<Object, Integer> identifiers = new HashMap<Object, Integer>();
		StringBuilder b = new StringBuilder();
		b.append("digraph LinkedList {\n");
		b.append("rankdir=LR;\n");
		int cnt = 0;
		ListNode p = head;
		while (p != null) {
			if (!identifiers.containsKey(p)) {
				identifiers.put(p, ++cnt);
				b.append(String.format("node%03d[label=\"%s\"];%n", cnt, p.value));
			}
			p = p.next;
		}
		if (head != null) {
			p = head;
			while (p.next != null) {
				final int pId = identifiers.get(p);
				final int nextId = identifiers.get(p.next);
				b.append(String.format("node%03d->node%03d%n", pId, nextId));
				b.append(String.format("node%03d->node%03d%n", nextId, pId));
				p = p.next;
			}
		}
		b.append("}");
		return b.toString();
	}

	/**
	 * Creates a dot string for a ring list.
	 */
	public static String makeDotForRingList(ListNode head) {
		final Map<Object, Integer> identifiers = new HashMap<Object, Integer>();
		StringBuilder b = new StringBuilder();
		b.append("digraph LinkedList {\n");
		b.append("rankdir=LR;\n");
		if (head != null) {
			int cnt = 0;
			ListNode p = head;
			do {
				if (!identifiers.containsKey(p)) {
					identifiers.put(p, ++cnt);
					b.append(String.format("node%03d[label=\"%s\"];%n", cnt, p.value));
				}
				p = p.next;
			} while (p != head);
			p = head;
			do {
				b.append(String.format("node%03d->node%03d%n", identifiers.get(p), identifiers.get(p.next)));
				p = p.next;
			} while (p != head);
		}
		b.append("}");
		return b.toString();
	}

	/**
	 * Creates a dot string which represents the heap.
	 */
	public static String makeDotForHeap(char[] values) {
		StringBuilder b = new StringBuilder();
		b.append("digraph Heap {\n");
		for (int i = 1; i < values.length; i++) {
			if (values[i] == 0) {
				throw new IllegalArgumentException(
						"Array must not contain null values." + " Please correct array length using Arrays.copyOf.");
			}
			String name = String.format("node%03d[label=\"%s\"]", i, values[i]);
			b.append(name + ";\n");
			if ((i * 2) < values.length) {
				b.append(String.format("node%03d", i));
				b.append("->");
				b.append(String.format("node%03d", i * 2) + ";");
				b.append("\n");
				if ((i * 2 + 1) < values.length) {
					b.append(String.format("node%03d", i));
					b.append("->");
					b.append(String.format("node%03d", i * 2 + 1) + ";");
					b.append("\n");
				}
			}
		}
		b.append("}");
		return b.toString();
	}

	/**
	 * Creates a dot string which represents the heap.
	 */
	public static String makeDotForHeap(Object[] values) {
		StringBuilder b = new StringBuilder();
		b.append("digraph Heap {\n");
		for (int i = 1; i < values.length; i++) {
			if (values[i] == null) {
				throw new IllegalArgumentException(
						"Array must not contain null values." + " Please correct array length using Arrays.copyOf.");
			}
			String name = String.format("node%03d[label=\"%s\"]", i, values[i].toString());
			b.append(name + ";\n");
			if ((i * 2) < values.length) {
				b.append(String.format("node%03d", i));
				b.append("->");
				b.append(String.format("node%03d", i * 2) + ";");
				b.append("\n");
				if ((i * 2 + 1) < values.length) {
					b.append(String.format("node%03d", i));
					b.append("->");
					b.append(String.format("node%03d", i * 2 + 1) + ";");
					b.append("\n");
				}
			}
		}
		b.append("}");
		return b.toString();
	}

	/**
	 * Creates a dot string for a graph.
	 */
	public static String makeDotForGraph(Graph graph) {
		return new DotMakerGraph(graph).getDot();
	}

	/**
	 * Creates a dot string for a hashtable.
	 */
	public static String makeDotForHashtable(Element[] hashtable) {
		return new DotMakerHashtable(hashtable).getDot();
	}

	/**
	 * Creates a dot string for a hashtable.
	 */
	public static String makeDotForHashtable(Element[] hashtable, float width) {
		return new DotMakerHashtable(hashtable, width).getDot();
	}

}
